selector_to_html = {"a[href=\"#identify-degrees-of-freedom\"]": "<h3 class=\"tippy-header\" style=\"margin-top: 0;\">1. Identify degrees of freedom<a class=\"headerlink\" href=\"#identify-degrees-of-freedom\" title=\"Link to this heading\">#</a></h3><p>The <code class=\"docutils literal notranslate\"><span class=\"pre\">Node</span></code> class automatically keeps track of the amount of degrees of freedom based on the amount of instances created with that class. For every node it will count 3 degrees of freedom. So far you\u2019ve only seen one degree of freedom per node, but soon more will follow!</p>", "a[href=\"#initialize-the-system-with-zeros\"]": "<h3 class=\"tippy-header\" style=\"margin-top: 0;\">2. Initialize the system with zeros<a class=\"headerlink\" href=\"#initialize-the-system-with-zeros\" title=\"Link to this heading\">#</a></h3><p>As said before, the global stiffness matrix, nodal displacement vector and nodal force vector are not defined anywhere in each of the classes. Therefore, you\u2019ll to create empty matrices and vectors yourself in the form of numpy arrays.</p>", "a[href=\"#assemble-stiffness-element-by-element\"]": "<h3 class=\"tippy-header\" style=\"margin-top: 0;\">3. Assemble stiffness, element by element<a class=\"headerlink\" href=\"#assemble-stiffness-element-by-element\" title=\"Link to this heading\">#</a></h3><p>The <code class=\"docutils literal notranslate\"><span class=\"pre\">Element</span></code> class can create a local stiffness matrix based on the <code class=\"docutils literal notranslate\"><span class=\"pre\">Node</span></code>-instances which are provided as an input. You\u2019ll need some smart indexing to map the local stiffness matrix to the correct location of the global stiffness matrix. Again, this is not part of any of the provided classes.</p>", "a[href=\"directly.html\"]": "<h1 class=\"tippy-header\" style=\"margin-top: 0;\">Setting up global equations directly<a class=\"headerlink\" href=\"#setting-up-global-equations-directly\" title=\"Link to this heading\">#</a></h1><p>Now that you\u2019ve seen the final matrix formulation, let\u2019s set up a procedure to find it directly!</p>", "a[href=\"#apply-external-loads\"]": "<h3 class=\"tippy-header\" style=\"margin-top: 0;\">4. Apply external loads<a class=\"headerlink\" href=\"#apply-external-loads\" title=\"Link to this heading\">#</a></h3><p>Loads can be added to the instances of the <code class=\"docutils literal notranslate\"><span class=\"pre\">Node</span></code> class. Together with the indexing of degrees of freedom inside these instances, the loads can be mapped to the correct location in the global nodal force vector.</p>", "a[href=\"#setting-up-global-equations-with-python-package\"]": "<h2 class=\"tippy-header\" style=\"margin-top: 0;\">Setting up global equations with python package<a class=\"headerlink\" href=\"#setting-up-global-equations-with-python-package\" title=\"Link to this heading\">#</a></h2><p>The steps provide in the <a class=\"reference internal\" href=\"directly.html\"><span class=\"std std-doc\">previous page</span></a> can be implemented using the matrix method package as follows:</p>", "a[href=\"#solve-for-the-unknown-nodal-displacements\"]": "<h3 class=\"tippy-header\" style=\"margin-top: 0;\">6. Solve for the unknown nodal displacements<a class=\"headerlink\" href=\"#solve-for-the-unknown-nodal-displacements\" title=\"Link to this heading\">#</a></h3><p>Again, the <code class=\"docutils literal notranslate\"><span class=\"pre\">Constrain</span></code> class can help you here. As it knows which degrees of freedom are fixed, it can give you a reduced global stiffness matrix and global nodal force vector which you can use yourself to solve for the free global nodal displacements</p>", "a[href=\"#apply-prescribed-displacements\"]": "<h3 class=\"tippy-header\" style=\"margin-top: 0;\">5. Apply prescribed displacements<a class=\"headerlink\" href=\"#apply-prescribed-displacements\" title=\"Link to this heading\">#</a></h3><p>The prescribed displacements can be applied using the <code class=\"docutils literal notranslate\"><span class=\"pre\">Constrain</span></code> class. It allows you to fix the degrees of freedom, which you\u2019ll need in the next step. It doesn\u2019t take care of the reaction force belonging to the prescribed displacement.</p>", "a[href=\"#code\"]": "<figure class=\"align-center\" id=\"code\">\n<img alt=\"../_images/code.svg\" src=\"../_images/code.svg\"/>\n<figcaption>\n<p><span class=\"caption-number\">Fig. 10 </span><span class=\"caption-text\">Structure of python package</span><a class=\"headerlink\" href=\"#code\" title=\"Link to this image\">#</a></p>\n</figcaption>\n</figure>", "a[href=\"#implementation-in-python\"]": "<h1 class=\"tippy-header\" style=\"margin-top: 0;\">Implementation in Python<a class=\"headerlink\" href=\"#implementation-in-python\" title=\"Link to this heading\">#</a></h1><p>The matrix method is implemented in an (incomplete) python package which you\u2019ll extend in the workshops and graded assignments.</p>"}
skip_classes = ["headerlink", "sd-stretched-link"]

window.onload = function () {
    for (const [select, tip_html] of Object.entries(selector_to_html)) {
        const links = document.querySelectorAll(`main ${select}`);
        for (const link of links) {
            if (skip_classes.some(c => link.classList.contains(c))) {
                continue;
            }

            tippy(link, {
                content: tip_html,
                allowHTML: true,
                arrow: false,
                placement: 'auto-start', maxWidth: 500, interactive: true, boundary: document.body, appendTo: document.body,
                onShow(instance) {MathJax.typesetPromise([instance.popper]).then(() => {});},
            });
        };
    };
    console.log("tippy tips loaded!");
};
